Page
Library
Module
Module type
Parameter
Class
Class type
Ocolor_printfThese functions are useful when trying to print with styles with Printf. However, they are a very bad choice. They perform a context-free formatting. For instance, there is no way to end the current style and restore the previous, like stack-base functions of Ocolor_format do. They just print the desired style and keep no track of the current state.
Thus, it's better to use Ocolor_format as much as possible. Moreover, Format provide a nice way of composing printers with %a and Format.asprintf. Even if you don't like boxes, Format.asprintf and semantic tags are cool.
So, here are the Printf-friendly functions. Use with care, and preferably, not at all.
val apply_style : out_channel -> Ocolor_types.style -> unitval apply_styles : out_channel -> Ocolor_types.style list -> unitval fg_color4 : out_channel -> Ocolor_types.color4 -> unitval fg_color8 : out_channel -> Ocolor_types.color8 -> unitval fg_color24 : out_channel -> Ocolor_types.color24 -> unitval bg_color4 : out_channel -> Ocolor_types.color4 -> unitval bg_color8 : out_channel -> Ocolor_types.color8 -> unitval bg_color24 : out_channel -> Ocolor_types.color24 -> unitval default_fg : out_channel -> unitval fg_rgb : int -> int -> int -> out_channel -> unitval default_bg : out_channel -> unitval bg_rgb : int -> int -> int -> out_channel -> unitval reset : out_channel -> unitval s_apply_style : unit -> Ocolor_types.style -> stringval s_apply_styles : unit -> Ocolor_types.style list -> stringJust like printf but perform automatic reset after printing if Ocolor_config.auto_reset is set
val fprintf : out_channel -> ('a, out_channel, unit) format -> 'aval kfprintf :
(out_channel -> unit) ->
out_channel ->
('a, out_channel, unit) format ->
'aval sprintf : ('a, unit, string, string) format4 -> 'aval printf : ('a, out_channel, unit) format -> 'aval kprintf : (out_channel -> unit) -> ('a, out_channel, unit) format -> 'amodule StylingPrettyPrinters : sig ... endinclude module type of struct include StylingPrettyPrinters endtype !'a pp = out_channel -> 'a -> unitval pp_bool_generic :
?false_style:Ocolor_types.style list ->
?true_style:Ocolor_types.style list ->
bool ppval pp_bool : bool ppval pp_option_generic :
?none:string ->
?none_style:Ocolor_types.style list ->
?some_style:Ocolor_types.style list ->
'a pp ->
'a option ppval pp_pair_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
'a pp ->
'b pp ->
('a * 'b) ppval pp_3_tuple_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
'a pp ->
'b pp ->
'c pp ->
('a * 'b * 'c) ppval pp_4_tuple_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
'a pp ->
'b pp ->
'c pp ->
'd pp ->
('a * 'b * 'c * 'd) ppval pp_5_tuple_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
'a pp ->
'b pp ->
'c pp ->
'd pp ->
'e pp ->
('a * 'b * 'c * 'd * 'e) ppval pp_iterable_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
(('a -> unit) -> 'b -> unit) ->
'a pp ->
'b ppval pp_list_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?elem_style:Ocolor_types.style list ->
'a pp ->
'a list ppval pp_iterable_mapping_more_generic :
?left:string ->
?sep:string ->
?right:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
(('a -> 'b -> unit) -> 'c -> unit) ->
('a * 'b) pp ->
'c ppval pp_iterable_mapping_generic :
?left:string ->
?sep:string ->
?right:string ->
?mapsto:string ->
?delim_style:Ocolor_types.style list ->
?sep_style:Ocolor_types.style list ->
?mapsto_style:Ocolor_types.style list ->
?key_style:Ocolor_types.style list ->
?value_style:Ocolor_types.style list ->
(('a -> 'b -> unit) -> 'c -> unit) ->
'a pp ->
'b pp ->
'c ppmodule NonStylingPrettyPrinters : sig ... end