Page
Library
Module
Module type
Parameter
Class
Class type
Ocolor_sgrSourceUntil the last moment, it is more convenient to work with list of integers. It is easier to apply several styles at once, since a sequence can be the concatenation of several simple sequences. And, at the last moment, we translate it as the string sequence as it will be passed to the terminal.
Translate a list of integer to the corresponding escape sequence string. This is the list of integer separated by Ocolor_config.separator (by default ";"), surrounded by the appropriate prefix and suffix (resp. "\x1b[" and "m")
Legal values for parameters are [0; 255].
Legal values for parameters are [0; 255].
Legal values for parameters are [1; 9]
int list