Page
Library
Module
Module type
Parameter
Class
Class type
Ocolor_typesSourceThe type of 4-bits colors. They are informally rgb colors where each component is in a scale from 0 to 1, with an additionnal intensity bit.
The type of rgb colors with each component in [0; 5]. This is used to encode the 6*6*6 color cube of 8-bits colors?
The type of a 8-bits color. Such a color is either a standard color, a 6*6*6 rgb color or some kind of grey on a 24-steps grayscale. Thus, the paramter of Grayscale should be in [0; 23]
The type of 24-bits colors. Each component belongs to [0; 255].
The type of a color that can be a 4-, 8- or 24-bits color.
type style = | Reset| Bold| Faint| Faint_bold_off| Italic| Italic_fraktur_off| Fraktur| DoubleUnderlined| Underlined| Underlined_off| Blink| Blink_off| Reverse_video| Reverse_video_off| Conceal| Conceal_off| Crossed_out| Crossed_out_off| Framed| Framed_encircled_off| Encircled| Overlined| Overlined_off| Default_font| Font of int| Fg of color| Default_fg| Bg of color| Default_bgAll style sequences that are implemented.
Reset disables everything (returns to default)Bold enables bold fontFaint decreases intensityFaint_bold_off revert the effect of Bold and FaintItalic: italic font (likely to not work)Fraktur: fraktur font (likely to not work)Italic_fraktur_off: disables italic and fraktur (likely to not work)Underlined: underlines textUnderlined_off: disable underlineBlink and Blink_off enables and disables flow blink (fast blink is rarely supported)Reverse_video and Reverse_video_off as expectedConceal and Conceal_off as excpected (rarely supported)Crossed_out and Crossed_out_off as excpectedFramed, Encircled and Framed_encircled_off as excpected (rarely supported)Overlined and Overlined_off as excpectedDefault_font set font to default (rarely supported)Font set font to specified number (in [1;9]) (rarely supported)Fg set foreground color to specified colorDefault_fg set foreground color to default colorBg set background color to specified colorDefault_bg set background color to default colortype color_palette = | VGA| CMD| Terminal_app| PuTTY| MIRC| Xterm| X| Ubuntu| Custom_palette of rgb Color4Map.tThe kind of palette the terminal uses. It is used to match a 4-bits color to a rgb color. This is useful to find the clostest 4-bits color to a rgb color, according to the actual colors.